#include "gtall.h"

/************************************************************************\
*
* Labels example part.
*
\************************************************************************/

#include "gtlogo.bm"

static GtAppShell *shell = NULL;
static int aligns[]={ GTA_CENTER, GTA_LEFT, GTA_RIGHT };
static char *strs[]=
{
	"center",
	"left",
	"right",
	NULL
};

static char *text[]=
{
	"\title",
	"Simple Labels",
	"The simple labels in this demo are from left to right:",
	"\nl",
	"1) A normal string label. One of the characters in the",
	"string is underlined as set by the 'underlinechar'",
	"resource. The underlinechar provide a single character",
	"and the first occurance of that character in the string",
	"is underlined. This can be used to indicate keyboard",
	"shortcuts for buttons and menu items.",
	"\nl",
	"2) A pixmap / bitmap label with a bitmap in this case.",
	"Only smaller pixmaps should be used in a label since the",
	"refreshing is not optimized for large label pixmaps.",
	"\nl",
	"3) A multi line label. Any string set in a label will",
	"be searched for newline characters to create multi",
	"lined labels. This label's alignment can be altered with",
	"the cycle widget to the right of it.",
	"\title",
	"Symbols",
	"The symbols in this demo is the ones avaliable in the",
	"GtSymbol class. Some of the symbols have different",
	"appearances based on a direction of on/off state.",
	"A symbol object is often created automatically as an",
	"indicator but it can always be suppressed to allow for",
	"an user defined object instead.",
	NULL
};

/************************************************************************\
*
* Close buttons shell but do not destroy it.
*
\************************************************************************/
static int CloseCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtAppShell *s;
	
	obj = obj;
	cd = cd;
	s = (GtAppShell *)ud;
	s->Map(FALSE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Select alignment for multi line label.
*
\************************************************************************/
static int AlignCB(GtBase *obj,GtCDPtr cd,GtUDPtr ud)
{
	GtCycleCD *ccd;
	GtLabel *l;
	
	obj = obj;
	ccd = (GtCycleCD *)cd;
	l = (GtLabel *)ud;
	l->SetV(GT_Align,aligns[ccd->nr],GT_DONE);
	return(GTCB_CONT);
}

/************************************************************************\
*
* Create labels shell.
*
\************************************************************************/
void CreateLabels(GtAppShell *parent)
{
	GtGroup *g1,*g2;
	GtMatrix *m;
	GtFrame *f;
	GtLabel *lab;
	GtCycle *c;
	GtButton *b;
	GtPixmap *pix;
	
	if(shell)
	{
		shell->Map(TRUE);
		return;
	}

	shell = GtMkAppShell("labels",parent,NULL);
	shell->AddCB(GT_Close,CloseCB,shell);
	
	g1 = GtMkGroup("vert",shell,
		GT_Orient,GTO_VERT,
		GT_DONE);
	
	f = GtMkFrame("simplefrm",g1,NULL);
	GtMkLabel("simple",f,GT_FrameTitle,TRUE,GT_DONE);
	
	g2 = GtMkGroup("simplegrp",f,
		GT_Orient,GTO_HORIZ,
		GT_DONE);

	GtMkLabel("label",g2,NULL);
	
	pix = GtMkPixmap(g2->GetDisp(),
		GT_BitmapData,gtlogo_bits,
		GT_XSize,gtlogo_width,
		GT_YSize,gtlogo_height,
		GT_DONE);
		
	GtMkLabel("gtlogo",g2,GT_Pixmap,pix,GT_DONE);

	lab = GtMkLabel("multiline",g2,GT_String,"Demo\nof\nGt Library",GT_DONE);
	
	c = GtMkCycle("aligns",g2,GT_LabelNames,strs,GT_DONE);
	c->AddCB(GT_Activate,AlignCB,lab);
	
	f = GtMkFrame("symsfrm",g1,NULL);
	GtMkLabel("syms",f,GT_FrameTitle,TRUE,GT_DONE);

	m = GtMkMatrix("matrix",f,
		GT_Columns,5,
		GT_Rows,4,
		GT_DONE);
		
	GtMkLabel("toggleradio",m,NULL);
	GtMkSymbol("tr1",m,GT_Symbol,GTST_TOGGLE,GT_OnOff,FALSE,GT_DONE);
	GtMkSymbol("tr2",m,GT_Symbol,GTST_TOGGLE,GT_OnOff,TRUE,GT_DONE);
	GtMkSymbol("tr3",m,GT_Symbol,GTST_RADIO,GT_OnOff,FALSE,GT_DONE);
	GtMkSymbol("tr4",m,GT_Symbol,GTST_RADIO,GT_OnOff,TRUE,GT_DONE);

	GtMkLabel("sarrow",m,NULL);
	GtMkSymbol("sa1",m,GT_Symbol,GTST_SARROW,GT_Direction,GTD_LEFT,GT_DONE);
	GtMkSymbol("sa2",m,GT_Symbol,GTST_SARROW,GT_Direction,GTD_UP,GT_DONE);
	GtMkSymbol("sa3",m,GT_Symbol,GTST_SARROW,GT_Direction,GTD_DOWN,GT_DONE);
	GtMkSymbol("sa4",m,GT_Symbol,GTST_SARROW,GT_Direction,GTD_RIGHT,GT_DONE);

	GtMkLabel("carrow",m,NULL);
	GtMkSymbol("ca1",m,GT_Symbol,GTST_CARROW,GT_Direction,GTD_LEFT,GT_DONE);
	GtMkSymbol("ca2",m,GT_Symbol,GTST_CARROW,GT_Direction,GTD_UP,GT_DONE);
	GtMkSymbol("ca3",m,GT_Symbol,GTST_CARROW,GT_Direction,GTD_DOWN,GT_DONE);
	GtMkSymbol("ca4",m,GT_Symbol,GTST_CARROW,GT_Direction,GTD_RIGHT,GT_DONE);

	GtMkLabel("cycle",m,NULL);
	GtMkSymbol("c1",m,GT_Symbol,GTST_CYCLE,GT_DONE);

	GtMkSeparator("sep",g1,NULL);
	
	b = GtMkButton("bok",g1,NULL);
	b->AddCB(GT_Activate,CloseCB,shell);
	GtMkLabel("ok",b,NULL);
	
	shell->Build();
}

/************************************************************************\
*
* Return the text for the labels part.
*
\************************************************************************/
char **GetLabelsText(void)
{
	return(text);
}
