#include <proto.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>

#include "gtappshell.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"

/************************************************************************\
*
* GtAppShell class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Type of key list elements */
typedef struct GtKeyList
{
	GtDLink lnk;
	KeySym ksym;
	ulong state;
	GtCallB Fun;
	GtUDPtr ud;
} GtKeyList;

/* Type of button list elements */
typedef struct GtButtonList
{
	GtDLink lnk;
	ulong button;
	ulong state;
	GtCallB Fun;
	GtUDPtr ud;
} GtButtonList;

/************************************************************************\
*
* Create and init appshell.
*
\************************************************************************/
GtAppShell *GtMkAppShell(char *n,GtManager *p,GtTag *tgs)
{
	GtAppShell *obj;
	
	if(!(obj=new GtAppShell()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtAppShell object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init appshell.
*
\************************************************************************/
GtAppShell *GtMkAppShell(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtAppShell *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkAppShell(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Create and init appshell with appdisp parent.
*
\************************************************************************/
GtAppShell *GtMkAppShell(char *n,GtAppDisp *p,GtTag *tgs)
{
	return(GtMkAppShell(n,(GtManager *)p,tgs));
}
	
/************************************************************************\
*
* Create and init appshell with appshell parent.
*
\************************************************************************/
GtAppShell *GtMkAppShell(char *n,GtAppShell *p,ulong tag,...)
{
	va_list va;
	GtAppShell *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkAppShell(n,(GtManager *)p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty appshell.
*
\************************************************************************/
GtAppShell::GtAppShell(void)
{
	GtDInit(&khd);
	GtDInit(&bhd);
	title=NULL;
	icon=NULL;
	geom=NULL;
	grp=0;
	xinc=0;
	yinc=0;
	delat=0;
	protat=0;
	clat=0;
	szht=NULL;
	cmaps=FALSE;
	iconic=FALSE;
	nominmax=FALSE;
}

/************************************************************************\
*
* Destroy appshell.
*
\************************************************************************/
void GtAppShell::Destroy(void)
{
	if(szht)
		XFree(szht);
	GtShell::Destroy();
}
	
/************************************************************************\
*
* Setup appshell.
*
\************************************************************************/
sbool GtAppShell::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	
	if(!GtShell::Init(n,p,tgs))
		return(FALSE);
	app->GetRes(this,GTR_TITLE,GTRT_STRING,&title);
	app->GetRes(this,GTR_ICON,GTRT_STRING,&icon);
	app->GetRes(this,GTR_GEOM,GTRT_STRING,&geom);
	app->GetRes(this,GTR_ICONIC,GTRT_BOOL,&iconic);
	app->GetRes(this,GTR_NOMINMAX,GTRT_BOOL,&nominmax);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Title:
				title=(char *)t->d.ptr;
				break;
			case GT_IconName:
				icon=(char *)t->d.ptr;
				break;
			case GT_Geometry:
				geom=(char *)t->d.ptr;
				break;
			case GT_XSizeInc:
				xinc=t->d.val;
				break;
			case GT_YSizeInc:
				yinc=t->d.val;
				break;
			case GT_Iconic:
				if(t->d.val)
					iconic=TRUE;
				else
					iconic=FALSE;
				break;
			case GT_NoMinMax:
				if(t->d.val)
					nominmax=TRUE;
				else
					nominmax=FALSE;
				break;
		}
	}
	if(!title)
	{
		if(parent)
			title=name;
		else
			app->GetV(GT_AppName,&title,GT_DONE);
	}
	if(!(delat=XInternAtom(dsp->XDsp(),"WM_DELETE_WINDOW",False)))
	{
		GtError(this,GTEC_NOXRESOURCE,"No delete atom!\n");
		return(FALSE);
	}
	if(!(protat=XInternAtom(dsp->XDsp(),"WM_PROTOCOLS",False)))
	{
		GtError(this,GTEC_NOXRESOURCE,"No protocols atom!\n");
		return(FALSE);
	}
	if(!(clat=XInternAtom(dsp->XDsp(),"WM_CLIENT_LEADER",False)))
	{
		GtError(this,GTEC_NOXRESOURCE,"No client leader atom!\n");
		return(FALSE);
	}
	return(TRUE);
}

/************************************************************************\
*
* Set appshell attributes.
*
\************************************************************************/
sbool GtAppShell::Set(GtTag *tgs)
{
	GtTag *t;
	sbool ch=FALSE;
	
	if(!GtShell::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Title:
				title=(char *)t->d.ptr;
				ch=TRUE;
				break;
		}
	}
	if(!title)
	{
		if(parent)
			title=name;
		else
			app->GetV(GT_AppName,&title,GT_DONE);
	}	
	if(ch)
		XStoreName(dsp->XDsp(),gtwin->XWin(),title);
	return(TRUE);
}

/************************************************************************\
*
* Get appshell attributes.
*
\************************************************************************/
void GtAppShell::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtShell::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Title:
				*ptr=(long)title;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtAppShell::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtAppShell")==0)
		return(TRUE);
	else
		return(GtShell::IsSubClassOf(cn));
}

/************************************************************************\
*
* Build window and add properties.
*
\************************************************************************/
sbool GtAppShell::Build(void)
{
	XClassHint *cht;
	XWMHints *wmht;
	int ac,xp,yp,xs,ys,gv;
	char **av=NULL;
	GtManager *w;
	Window wn;
	
	if(!(szht=XAllocSizeHints()))
	{
		GtError(this,GTEC_NOMEMORY,"No memory for size hints!\n");
		return(FALSE);
	}
	xp = xpos;
	yp = ypos;
	xs = xsize;
	ys = ysize;
	gv=XWMGeometry(dsp->XDsp(),dsp->XDefaultScreen(),geom,NULL,
									0,szht,&xp,&yp,&xs,&ys,&szht->win_gravity);
	if(gv & XValue)
	{
		szht->flags|=USPosition;
		szht->x=xp;
	}
	if(gv & YValue)
	{
		szht->flags|=USPosition;
		szht->y=yp;
	}
	if(gv & WidthValue)
	{
		szht->flags|=USSize|PBaseSize;
		szht->width=xs;
		szht->base_width=xs;
	}
	if(gv & HeightValue)
	{
		szht->flags|=USSize|PBaseSize;
		szht->height=ys;
		szht->base_height=ys;
	}
	if(!nominmax)
	{
		szht->flags|=PMinSize|PMaxSize;
		this->GetMinSize(&szht->min_width,&szht->min_height);
		this->GetMaxSize(&szht->max_width,&szht->max_height);
	}
	if(xinc && yinc)
	{
		szht->flags|=PResizeInc;
		szht->width_inc=xinc;
		szht->height_inc=yinc;
	}
	if(!(cht=XAllocClassHint()))
	{
		XFree(szht);
		GtError(this,GTEC_NOMEMORY,"No memory for class hints!\n");
		return(FALSE);
	}
	app->GetV(GT_AppName,&cht->res_name,GT_AppClass,&cht->res_class,GT_DONE);
	app->GetArgs(&ac,&av);
	if(!(wmht=XAllocWMHints()))
	{
		XFree(szht);
		XFree(cht);
		GtError(this,GTEC_NOMEMORY,"No memory for window manager hints!\n");
		return(FALSE);
	}
	wmht->flags=InputHint|StateHint;
	wmht->input=True;
	wmht->initial_state=(iconic ? IconicState : NormalState);
	if(grp)
	{
		wmht->flags|=WindowGroupHint;
		wmht->window_group=grp;
	}
	XmbSetWMProperties(dsp->XDsp(),gtwin->XWin(),title,icon,av,ac,szht,wmht,cht);
	XSetWMProtocols(dsp->XDsp(),gtwin->XWin(),&delat,1);
	XSetCommand(dsp->XDsp(),gtwin->XWin(),av,ac);
	this->UpdateColormaps();
	if(parent)
	{
		w=this->GetParent();
		while(w->GetParent())
			w=w->GetParent();
		wn=w->GetWin()->XWin();
	}
	else
		wn=gtwin->XWin();
	XChangeProperty(dsp->XDsp(),gtwin->XWin(),clat,XA_WINDOW,32,PropModeReplace,
										(ubyte *)&wn,1);
	XFree(cht);
	XFree(wmht);
	this->MoveResize(xp,yp,xs,ys);
	return(GtShell::Build());
}

/************************************************************************\
*
* Handle appshell events.
*
\************************************************************************/
int GtAppShell::Handle(XEvent *xe)
{
	int ret;
	
	ret=GtShell::Handle(xe);
	if(xe->type==KeyPress)
	{
		this->Key(xe);
		return(GTHR_USED);
	}
	else if((xe->type==ButtonPress) || (xe->type==ButtonRelease))
	{
		if(this->Mouse(xe))
			return(GTHR_USED);
	}
	return(ret);
}

/************************************************************************\
*
* Set appshell event mask.
*
\************************************************************************/
void GtAppShell::SetEMask(ulong em)
{
	GtShell::SetEMask(em|KeyPressMask|ButtonPressMask|ButtonReleaseMask);
}

/************************************************************************\
*
* Add callback.
*
\************************************************************************/
void GtAppShell::AddCB(ulong tag,GtCallB cb,GtUDPtr ud)
{
	if(tag==GT_Close)
		close.AddCB(cb,ud);
	else if(tag == GT_Key)
		key.AddCB(cb,ud);
	else
		GtShell::AddCB(tag,cb,ud);
}

/************************************************************************\
*
* Remove callback.
*
\************************************************************************/
void GtAppShell::RemCB(ulong tag,GtCallB cb)
{
	if(tag==GT_Close)
		close.RemCB(cb);
	else if(tag == GT_Key)
		key.RemCB(cb);
	else
		GtShell::RemCB(tag,cb);
}

/************************************************************************\
*
* Add key callback.
*
\************************************************************************/
int GtAppShell::AddKeyCB(char *kstr,GtCallB cb,GtUDPtr ud)
{
	GtKeyList *kl;
	
	if(!kstr)
		return(0);
	if(!(kl=(GtKeyList *)malloc(sizeof(GtKeyList))))
	{
		GtError(this,GTEC_NOMEMORY,"No memory for key callback!\n");
		return(0);
	}
	kl->state=0;
	if(strchr(kstr,'-'))
	{
		while(*kstr && (*kstr!='-'))
		{
			switch(*kstr)
			{
				case 'S':
					kl->state|=ShiftMask;
					break;
				case 'C':
					kl->state|=ControlMask;
					break;
				case 'A':
				case '1':
					kl->state|=Mod1Mask;
					break;
				case '2':
					kl->state|=Mod2Mask;
					break;
				case '3':
					kl->state|=Mod3Mask;
					break;
				case '4':
					kl->state|=Mod4Mask;
					break;
				case '5':
					kl->state|=Mod5Mask;
					break;
				case 'L':
					kl->state|=Button1Mask;
					break;
				case 'R':
					kl->state|=Button3Mask;
					break;
				case 'M':
					kl->state|=Button2Mask;
					break;
			}
			kstr++;
		}
		if(*kstr=='-')
			kstr++;
	}
	kl->ksym=XStringToKeysym(kstr);
	kl->Fun=cb;
	kl->ud=ud;
	GtDAddTail(&khd,&kl->lnk);
	return((int)kl);
}

/************************************************************************\
*
* Remove key callback.
*
\************************************************************************/
void GtAppShell::RemKeyCB(int id)
{
	GtKeyList *kl;
	
	for(kl=(GtKeyList *)GtDFirst(&khd); kl;
			kl=(GtKeyList *)GtDNext(&khd,&kl->lnk))
	{
		if(kl==(GtKeyList *)id)
		{
			GtDRemove(&kl->lnk);
			free(kl);
			return;
		}
	}
}

/************************************************************************\
*
* Add button callback.
*
\************************************************************************/
int GtAppShell::AddButtonCB(ulong button,ulong state,GtCallB cb,GtUDPtr ud)
{
	GtButtonList *bl;
	
	if(!(bl=(GtButtonList *)malloc(sizeof(GtButtonList))))
	{
		GtError(this,GTEC_NOMEMORY,"No memory for button callback!\n");
		return(0);
	}
	bl->button=button;
	bl->state=state;
	bl->Fun=cb;
	bl->ud=ud;
	GtDAddTail(&bhd,&bl->lnk);
	return((int)bl);
}

/************************************************************************\
*
* Remove button callback.
*
\************************************************************************/
void GtAppShell::RemButtonCB(int id)
{
	GtButtonList *bl;
	
	for(bl=(GtButtonList *)GtDFirst(&bhd); bl;
			bl=(GtButtonList *)GtDNext(&bhd,&bl->lnk))
	{
		if(bl==(GtButtonList *)id)
		{
			GtDRemove(&bl->lnk);
			free(bl);
			return;
		}
	}
}

/************************************************************************\
*
* Update colormaps property on window.
*
\************************************************************************/
void GtAppShell::UpdateColormaps(void)
{
	Window *wl=NULL;
	int wc;
	
	wc=this->CountColormaps((GtWList *)GtSFirst(&man));
	if(wc)
	{
		if(!(wl=(Window *)malloc(sizeof(Window *)*wc)))
		{
			GtError(this,GTEC_NOMEMORY,"No memory for colormap windows array!\n");
			return;
		}
		this->ColormapWindows((GtWList *)GtSFirst(&man),wl);
		XSetWMColormapWindows(dsp->XDsp(),gtwin->XWin(),wl,wc);
		cmaps=TRUE;
	}
	else if(cmaps)
	{
		wl=NULL;
		XSetWMColormapWindows(dsp->XDsp(),gtwin->XWin(),wl,wc);
	}
	if(wl)
		free(wl);
}

/************************************************************************\
*
* Handle appshell key events.
*
\************************************************************************/
sbool GtAppShell::Key(XEvent *xe)
{
	GtEventCD cd;
	GtKeyList *kl,*tmp;
	KeySym ks;
	sbool ret=FALSE;
	
	if(xe->type==KeyPress)
	{
		ks=XLookupKeysym(&xe->xkey,0);
		for(kl=(GtKeyList *)GtDFirst(&khd); kl; kl=tmp)
		{
			tmp=(GtKeyList *)GtDNext(&khd,&kl->lnk);
			if((kl->ksym==ks) && ((xe->xkey.state & kl->state)==kl->state))
			{
				if(kl->Fun(this,xe,kl->ud)==GTCB_BREAK)
					return(TRUE);
				ret=TRUE;
			}
		}
	}
	cd.xevent = xe;
	cd.used = ret;
	key.CallCB(this,&cd);
	return(cd.used);
}

/************************************************************************\
*
* Handle appshell mouse events.
*
\************************************************************************/
sbool GtAppShell::Mouse(XEvent *xe)
{
	GtButtonList *bl,*tmp;
	sbool ret=FALSE;
	
	if((xe->type==ButtonPress) || (xe->type==ButtonRelease))
	{
		for(bl=(GtButtonList *)GtDFirst(&bhd); bl; bl=tmp)
		{
			tmp=(GtButtonList *)GtDNext(&bhd,&bl->lnk);
			if((bl->button==xe->xbutton.button) && 
				((xe->xbutton.state & bl->state)==bl->state))
			{
				if(bl->Fun(this,xe,bl->ud)==GTCB_BREAK)
					return(TRUE);
				ret=TRUE;
			}
		}
	}
	return(ret);
}

/************************************************************************\
*
* Handle delete from window manager.
*
\************************************************************************/
sbool GtAppShell::Misc(XEvent *xe)
{
	sbool ret;
	
	ret=GtShell::Misc(xe);
	if(xe && (xe->type==ClientMessage) && (xe->xclient.message_type==protat) &&
		(xe->xclient.data.l[0]==(long)delat))
	{
		ret=TRUE;
		close.CallCB(this,xe);
	}
	return(ret);
}

/************************************************************************\
*
* Relayout and set minimum size in property if changed.
*
\************************************************************************/
void GtAppShell::Relayout(void)
{
	sbool ch=FALSE;
	
	if(flags & GTWF_UNBUILT)
		return;
	GtShell::Relayout();
	if(szht && ((minxs!=szht->min_width) || (minys!=szht->min_height)))
	{
		szht->min_width=minxs;
		szht->min_height=minys;
		ch=TRUE;
	}
	if(szht && ((maxxs!=szht->max_width) || (maxys!=szht->max_height)))
	{
		szht->max_width=maxxs;
		szht->max_height=maxys;
		ch=TRUE;
	}
	if(ch)
		XSetWMNormalHints(dsp->XDsp(),gtwin->XWin(),szht);
}

/************************************************************************\
*
* Count number of non-default colormaps.
*
\************************************************************************/
int GtAppShell::CountColormaps(GtWList *wl)
{
	GtColormap *cm=NULL;
	int ret=0;
	
	if(wl)
	{
		cm=(GtColormap *)wl->w->GetA(GT_Colormap,0);
		if(cm && (cm->GetA(GT_Mode,0)!=GTCM_DEFAULT))
			ret++;
		ret+=this->CountColormaps((GtWList *)GtSNext(&wl->lnk));
		if(wl->w->IsSubClassOf("GtManager"))
			ret+=this->CountColormaps(((GtManager *)wl->w)->GetManaged());
	}
	return(ret);
}

/************************************************************************\
*
* Fill in array with window containing non-default colormaps.
*
\************************************************************************/
Window *GtAppShell::ColormapWindows(GtWList *wl,Window *wp)
{
	GtColormap *cm=NULL;
	
	if(wl)
	{
		cm=(GtColormap *)wl->w->GetA(GT_Colormap,0);
		if(cm && (cm->GetA(GT_Mode,0)!=GTCM_DEFAULT))
		{
			*wp=wl->w->GetWin()->XWin();
			wp++;
		}
		wp=this->ColormapWindows((GtWList *)GtSNext(&wl->lnk),wp);
		if(wl->w->IsSubClassOf("GtManager"))
			wp=this->ColormapWindows(((GtManager *)wl->w)->GetManaged(),wp);
	}
	return(wp);
}
	
