#include <proto.h>

#include "gtcascade.h"
#include "gtsymbol.h"
#include "gtmenu.h"
#include "gtmenubar.h"
#include "gtapp.h"
#include "gterror.h"
#include "gtresname.h"
#include "gtcommon.h"
#include "gtconfig.h"

/************************************************************************\
*
* GtCascade class.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/************************************************************************\
*
* Create and init cascade.
*
\************************************************************************/
GtCascade *GtMkCascade(char *n,GtManager *p,GtTag *tgs)
{
	GtCascade *obj;
	
	if(!(obj=new GtCascade()))
	{
		GtError(NULL,GTEC_NOOBJECT,"No GtCascade object!\n");
		return(NULL);
	}
	if(!obj->Init(n,p,tgs))
	{
		GtDestroy(obj);
		obj=NULL;
	}
	return(obj);
}
	
/************************************************************************\
*
* Create and init cascade.
*
\************************************************************************/
GtCascade *GtMkCascade(char *n,GtManager *p,ulong tag,...)
{
	va_list va;
	GtCascade *obj;
	int ps;
	
	va_start(va,tag);
	ps=GtBeginVaTags();
	obj=GtMkCascade(n,p,GtCopyVaTags(tag,&va));
	GtEndVaTags(ps);
	va_end(va);
	return(obj);
}

/************************************************************************\
*
* Setup empty cascade.
*
\************************************************************************/
GtCascade::GtCascade(void)
{
	menu=NULL;
	dir=GTD_DOWN;
	ind=NULL;
}

/************************************************************************\
*
* Setup cascade.
*
\************************************************************************/
sbool GtCascade::Init(char *n,GtManager *p,GtTag *tgs)
{
	GtTag *t;
	char *iname="cascind";
	long pos=0;
	
	if(!GtButton::Init(n,p,tgs))
		return(FALSE);
	if(mshell && !mshell->IsSubClassOf("GtMenuBar"))
		dir=GTD_RIGHT;
	app->GetRes(this,GTR_DIR,GTRT_DIR,&dir);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Menu:
				menu=(GtMenu *)t->d.ptr;
				break;
			case GT_Direction:
				dir=t->d.val;
				if(dir>GTD_LAST_NR)
					dir=GTD_DOWN;
				break;
			case GT_SymbolName:
				iname=(char *)t->d.ptr;
				break;
		}
	}
	bflags&=~GTBF_REPEAT;
	if(iname && (!mshell || !mshell->IsSubClassOf("GtMenuBar")))
	{
		if(dir==GTD_RIGHT)
			pos=GTP_LAST;
		if(!(ind=GtMkSymbol(iname,this,GT_Symbol,GTST_SARROW,GT_Position,pos,
																		GT_Direction,dir,GT_DONE)))
			return(FALSE);
	}
	return(TRUE);
}

/************************************************************************\
*
* Set toggle attributes.
*
\************************************************************************/
sbool GtCascade::Set(GtTag *tgs)
{
	GtTag *t;
	
	if(!GtButton::Set(tgs))
		return(FALSE);
	while((t=GtNextTag(&tgs)))
	{
		switch(t->tag)
		{
			case GT_Menu:
				menu=(GtMenu *)t->d.ptr;
				break;
		}
	}
	bflags&=~GTBF_REPEAT;
	return(TRUE);
}

/************************************************************************\
*
* Get toggle attributes.
*
\************************************************************************/
void GtCascade::Get(GtTag *tgs)
{
	GtTag *t;
	long *ptr;
	
	GtButton::Get(tgs);
	while((t=GtNextTag(&tgs)))
	{
		if(!(ptr=(long *)t->d.ptr))
			continue;
		switch(t->tag)
		{
			case GT_Menu:
				*ptr=(long)menu;
				break;
			case GT_Direction:
				*ptr=dir;
				break;
		}
	}
}

/************************************************************************\
*
* Check if class is (sub)class of named class.
*
\************************************************************************/
sbool GtCascade::IsSubClassOf(char *cn)
{
	if(strcmp(cn,"GtCascade")==0)
		return(TRUE);
	else
		return(GtButton::IsSubClassOf(cn));
}

/************************************************************************\
*
* Keep childrean to the right/left of indicator.
*
\************************************************************************/
sbool GtCascade::AddChild(GtWidget *obj,ulong pos)
{
	sbool ret=FALSE;
	
	if((dir!=GTD_RIGHT) && ind)
	{
		ret=GtButton::AddChild(obj,GTMAX(pos,1));
	}
	else if((dir==GTD_RIGHT) && ind)
	{
		ret=GtButton::AddChild(obj,GTMIN(pos,(ulong)(nrchild-1)));
	}
	else
	{
		ret=GtButton::AddChild(obj,pos);
	}
	if(!ind && obj->IsSubClassOf("GtSymbol"))
		ind=(GtSymbol *)obj;
	return(ret);
}

/************************************************************************\
*
* Check for activate cascade notify.
*
\************************************************************************/
void GtCascade::Notify(ulong tag)
{
	if(tag!=GT_Activate)
		GtButton::Notify(tag);
}

/************************************************************************\
*
* Notify menu popped down.
*
\************************************************************************/
void GtCascade::NotifyA(ulong tag,void *ptr)
{
	ubyte tmp;
	
	GtButton::NotifyA(tag,ptr);
	if(tag==GT_Popdown)
	{
		tmp=bflags & GTBF_ARMED;
		bflags&=~(GTBF_ARMED|GTBF_PRESS);
		this->Refresh();
		if(tmp)
			darm.CallCB(this,NULL);
	}
}

/************************************************************************\
*
* Mouse button.
*
\************************************************************************/
sbool GtCascade::Mouse(XEvent *xe)
{
	int b,s;
	ubyte tmp;
	
	if(!xe)
		return(FALSE);
	switch(xe->type)
	{
		case ButtonPress:
			b=xe->xbutton.button;
			if(flags & GTWF_DISABLED)
				break;
			if(!(bflags & GTBF_PRESS) && (b==GTDEF_MENUBUTTON))	
			{
				bflags|=(GTBF_ARMED|GTBF_PRESS);
				gtime=xe->xbutton.time;
				this->Refresh();
				this->Arm(b,xe->xbutton.state);
				return(TRUE);
			}
			break;
		case ButtonRelease:
			b=xe->xbutton.button;
			if((bflags & GTBF_PRESS) && (b==GTDEF_MENUBUTTON))
			{
				tmp=bflags & GTBF_ARMED;
				if(!(mshell && (b==Button1)))
					bflags&=~(GTBF_ARMED|GTBF_PRESS);
				gtime=xe->xbutton.time;
				this->Refresh();
				if(tmp)
				{
					this->Activate(b,xe->xbutton.state);
				}
				return(TRUE);
			}
			break;
		case EnterNotify:
			if((flags & GTWF_DISABLED) || (xe->xcrossing.mode!=NotifyNormal))
				break;
			s=xe->xcrossing.state;
			if(mshell && (s & GTDEF_MENUBMASK))
				bflags|=GTBF_PRESS;
			if((bflags & GTBF_PRESS) && !(bflags & GTBF_ARMED))
			{
				if(mshell)
					mshell->NotifyA(GT_Popdown,(void *)CurrentTime);
				bflags|=GTBF_ARMED;
				gtime=xe->xcrossing.time;
				this->Refresh();
				this->Arm(0,s);
				return(TRUE);
			}
			break;
	}
	return(FALSE);
}


/************************************************************************\
*
* Arm cascade button.
*
\************************************************************************/
void GtCascade::Arm(ulong button,ulong state)
{
	button=button;
	state=state;
	arm.CallCB(this,NULL);
	if(mshell && menu)
	{
		if(mshell->IsSubClassOf("GtMenu"))
			mshell->PopupSub(this,menu,gtime);
		else if(mshell->IsSubClassOf("GtMenuBar"))
			((GtMenuBar *)mshell)->PopupSub(this,menu,gtime);
	}
	if(menu)
		menu->Popup(this,GTDEF_MENUBUTTON,gtime);
}

/************************************************************************\
*
* Activate toggle button.
*
\************************************************************************/
void GtCascade::Activate(ulong button,ulong state)
{
	button=button;
	state=state;
	if(mshell && menu)
	{
		mshell->NotifyA(GT_Popdown,(void *)gtime);
/*
		if(mshell->IsSubClassOf("GtMenu"))
			mshell->PopdownSub(gtime);
		else if(mshell->IsSubClassOf("GtMenuBar"))
			((GtMenuBar *)mshell)->PopdownSub();
*/
	}
	else if(menu)
		menu->Popdown();
	darm.CallCB(this,NULL);
}

/************************************************************************\
*
* Disarm toggle button.
*
\************************************************************************/
void GtCascade::DisArm(ulong button,ulong state)
{
	button=button;
	state=state;
	if(mshell && menu)
	{
		mshell->NotifyA(GT_Popdown,(void *)gtime);
/*
		if(mshell->IsSubClassOf("GtMenu"))
			mshell->PopdownSub(gtime);
		else if(mshell->IsSubClassOf("GtMenuBar"))
			((GtMenuBar *)mshell)->PopdownSub();
*/
	}
	else if(menu)
		menu->Popdown();
	darm.CallCB(this,NULL);
}
